;; Extremely simple args handling for xattr

(module xattr-args
 (parse-arguments option args-error)
(import scheme chicken data-structures)
(require-library srfi-37)
(import (prefix srfi-37 srfi-37:))

;; Disgusting hack
(define args-error (make-parameter error))

;; SRFI-37 reference implementation is fairly broken.

;; - Argument value for non-arg options is #t (as opposed to #f in srfi-37);
;; value for optional-arg options with missing arg is #f.
;; - Aliased options '(#\v #\V "verbose") are normalized to the car, #\v.
(define (cons-option option name arg options operands)
  (define (->symbol x)
    (if (string? x) (string->symbol x) x))
  ;; Required can only be detected if it's the last one on the command-line due to SRFI-37
  (when (and (not arg)
             (srfi-37:option-required-arg? option))
    ;; FIXME: bad
    ((args-error) (string-append "option requires an argument: " name)))
  (values (cons (cons (->symbol (car (srfi-37:option-names option)))
                      (if (srfi-37:option-optional-arg? option)
                          arg
                          (or arg #t)))
                options)
          operands))

(define (cons-operand operand options operands)
  (values options
          (cons operand operands)))

(define (symbol->keyword k)
  (cond ((keyword? k) k)
        ((symbol? k)
         (string->keyword (symbol->string k)))
        (else k)))

;; accepted for type: #:required, 'required, #t -> required option
;;                    #f -> no argument
;;                    #:optional, 'optional -> optional argument
(define (option names #!optional (type #f) (processor #f))
  (let ((type (symbol->keyword type))
        (processor (or processor cons-option)))
    (srfi-37:option names
                    (or (eq? type #:required)
                        (eq? type #t))
                    (eq? type #:optional)
                    processor)))

(define (parse-arguments args opts)
 (receive (opt oper)
     (srfi-37:args-fold args opts
      (lambda (option name args . seeds)
        ;; FIXME: bad
        ((args-error) (string-append "unrecognized option: ") (->string name)))
      cons-operand
      '()
      '())
   (list
    (reverse opt)      ; skip for alist?
    (reverse oper))))
)
